import logging

from src.senseid.parsers.rain import SenseidRainTag

logging.basicConfig(level=logging.DEBUG)

# Parse from EPC string
tag = SenseidRainTag('000000F1D301010000012301')
logging.info(tag)

# Parse from EPC bytearray
tag = SenseidRainTag(bytearray([0x00, 0x00, 0x00, 0xF1, 0xD3, 0x01, 0x01, 0x00, 0x00, 0x01, 0x23, 0x01]))
logging.info(tag)

# Convert to dict
tag_dict = tag.to_dict()
logging.info(tag_dict)

# Convert to json
tag_json = tag.to_json(indent=2)
logging.info(tag_json)


import logging
import time
from threading import Thread
from typing import Callable

import serial
import serial.tools.list_ports

from .. import SenseidReaderConnectionInfo, SupportedSenseidReader

logger = logging.getLogger(__name__)


class SerialPortScanner:

    def __init__(self, notification_callback: Callable[[SenseidReaderConnectionInfo], None]):
        self.notification_callback = notification_callback
        self._scan_thread = None
        self.comports = []
        self._is_on = False

    def start(self, reset: bool = False):
        if reset:
            self.comports = []
        self._is_on = True
        self._scan_thread = Thread(target=self._scan_job, daemon=True)
        self._scan_thread.start()

    def stop(self):
        self._is_on = False
        self._scan_thread.join()

    def _scan_job(self):
        while self._is_on:
            com_port_list = serial.tools.list_ports.comports()
            for com_port in com_port_list:
                # REDRCP
                #if 'Silicon Lab' in str(com_port.manufacturer):
                if 'VID:PID=10C4:EA60' in str(com_port.hwid):
                    if com_port.name not in self.comports:
                        logger.info('New REDRCP reader found: ' + com_port.name)
                        self.comports.append(com_port.name)
                        self.notification_callback(SenseidReaderConnectionInfo(driver=SupportedSenseidReader.REDRCP,
                                                                               connection_string=com_port.name))
                # NUR
                #if 'NUR Module' in str(com_port.manufacturer):
                if 'VID:PID=04E6:0112' in str(com_port.hwid):
                    if com_port.name not in self.comports:
                        logger.info('New NUR reader found: ' + com_port.name)
                        self.comports.append(com_port.name)
                        self.notification_callback(SenseidReaderConnectionInfo(driver=SupportedSenseidReader.NURAPI,
                                                                               connection_string=com_port.name))
                        self.notification_callback(SenseidReaderConnectionInfo(driver=SupportedSenseidReader.NURAPY,
                                                                               connection_string=com_port.name))
            time.sleep(1)



from dataclasses import dataclass
from enum import Enum
from typing import List

from dataclasses_json import dataclass_json


class SenseidTechnologies(Enum):
    RAIN = 'RAIN'
    BLE = 'BLE'
    NFC = 'NFC'


@dataclass_json
@dataclass
class SenseidData:
    magnitude: str
    unit_long: str
    unit_short: str
    value: float


@dataclass_json
@dataclass
class SenseidTag:
    technology: SenseidTechnologies
    fw_version: int
    sn: int
    id: str
    name: str
    description: str
    data: List[SenseidData] | None



import logging
from typing import List, Callable

from redrcp import RedRcp, NotificationTpeCuiii, NotificationTpeCuiiiRssi, NotificationTpeCuiiiTid, AntiCollisionMode, \
    ParamDR, ParamModulation, ParamSel, ParamSession, ParamTarget

from ..parsers import SenseidTag
from ..parsers.rain import SenseidRainTag
from ..readers import SenseidReader, SenseidReaderDetails


class SenseidReaderRedRcp(SenseidReader):

    def __init__(self):
        self.driver = RedRcp()
        self.notification_callback = None
        self.details = None

    def connect(self, connection_string: str):
        if not self.driver.connect(connection_string=connection_string):
            return False
        self.driver.set_notification_callback(self._redrcp_notification_callback)
        self.driver.set_anti_collision_mode(AntiCollisionMode.MANUAL, start_q=4, min_q=0, max_q=7)
        self.driver.set_query_parameters(dr=ParamDR.DR_64_DIV_3,
                                         modulation=ParamModulation.MILLER_4,
                                         pilot_tone=False,
                                         sel=ParamSel.ALL_0,
                                         session=ParamSession.S1,
                                         target=ParamTarget.A,
                                         target_toggle=True,
                                         q=4)
        return True

    def _redrcp_notification_callback(self, red_rcp_notification: NotificationTpeCuiii |
                                                                  NotificationTpeCuiiiRssi |
                                                                  NotificationTpeCuiiiTid):
        if self.notification_callback is not None:
            self.notification_callback(SenseidRainTag(epc=red_rcp_notification.epc))

    def disconnect(self):
        self.driver.disconnect()

    def get_details(self):
        info_model = self.driver.get_info_model()
        info_fw_version = self.driver.get_info_fw_version()
        info_detail = self.driver.get_info_detail()
        self.details = SenseidReaderDetails(
            model_name=info_model,
            region=info_detail.region.name,
            firmware_version=info_fw_version,
            antenna_count=1,
            min_tx_power=info_detail.min_tx_power,
            max_tx_power=info_detail.max_tx_power
        )
        return self.details

    def get_tx_power(self):
        return self.driver.get_tx_power()

    def set_tx_power(self, dbm):
        if self.details is None:
            self.get_details()
        if dbm > self.details.max_tx_power:
            dbm = self.details.max_tx_power
            logging.warning('Power set to max power: ' + str(dbm))
        if dbm < self.details.min_tx_power:
            dbm = self.details.min_tx_power
            logging.warning('Power set to min power: ' + str(dbm))
        self.driver.set_tx_power(dbm=dbm)

    def get_antenna_config(self):
        # RED4S has a single antenna
        antenna_config_array: List[bool] = [True]
        return antenna_config_array

    def set_antenna_config(self, antenna_config_array: List[bool]):
        # RED4S has a single antenna
        pass

    def start_inventory_async(self, notification_callback: Callable[[SenseidTag], None]):
        self.notification_callback = notification_callback
        return self.driver.start_auto_read2()

    def stop_inventory_async(self):
        if self.driver.is_connected():
            return self.driver.stop_auto_read2()
